# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tools for manipulating command line arguments"""

import logging
import re


logging.getLogger(__name__).addHandler(logging.NullHandler())


class Range:
    """Class accepts a range specification from the command line

    Parameter:
        range_spec: String indicating the range spec

    RANGEs are prefixed with @ and specified 'min:max' or 'min:' or ':max'
    (or 'max'). If specified 'max:min', a warning status will be generated
    if the count is inside the specified range

    Single values indicate  the upper

    This plugin checks the number of currently running processes and
    generates WARNING or CRITICAL states if the process count is outside
    the specified threshold ranges. The process count can be filtered by
    process owner, parent process PID, current state (e.g., 'Z'), or may
    be the total number of running processes

    """

    RE_RANGE = re.compile(
        "^@"
        "(?P<lval>-?(?:[0-9]+[.])?[0-9]+)?"
        ":"
        "(?P<rval>-?(?:[0-9]+[.])?[0-9]+)?"
        "$"
    )
    RE_SINGLE = re.compile("^[0-9]+$")
    CONV = int
    MIN = 0
    MAX = 2**32 - 1

    def __init__(self, range_spec: str):
        self._range_spec = range_spec
        self._low_bounds = None
        self._high_bounds = None
        self._inverse = False

        log = logging.getLogger(f"{__name__}.{__class__.__name__}")
        log.debug("Range Spec: %r", range_spec)

        found = False
        mobj = self.RE_RANGE.match(range_spec)
        if mobj:  # Range found
            log.debug("Range found")
            self._range_parse(mobj)
            found = True

        if not found:
            if self.RE_SINGLE.match(range_spec):
                log.debug("Single value found")
                self._low_bounds = self.CONV(0)
                self._high_bounds = self.CONV(range_spec)
                found = True

        if not found:
            raise ValueError(f"Invalid range spec detected: {range_spec}")

    def _range_parse(self, mobj: re.Match):
        """Parse ranges"""
        log = logging.getLogger(
            f"{__name__}.{__class__.__name__}._range_parse"
        )
        try:
            lval = mobj.group("lval")
            if lval:
                lval = self.CONV(lval)
            else:
                lval = self.CONV(self.MIN)
        except TypeError as err:
            raise TypeError(
                f"Invalid conversion for left side of range: {err}"
            ) from None
        try:
            rval = mobj.group("rval")
            if rval:
                rval = self.CONV(rval)
            else:
                rval = self.CONV(self.MAX)
        except TypeError as err:
            raise TypeError(
                f"Invalid conversion for right side of range: {err}"
            ) from None

        if lval > rval:  # max:min
            log.debug("Inverse found: lval=%r rval=%r", lval, rval)
            self._inverse = True
            self._low_bounds = rval
            self._high_bounds = lval
        elif lval <= rval:  # min: min:max or :max
            log.debug("Normal bounds found: lval=%r rval=%r", lval, rval)
            self._low_bounds = lval
            self._high_bounds = rval
        else:
            # Never should be hit
            raise ValueError(f"Fatal Error parsing value: {self._range_spec}")

    def __contains__(self, value: int | float):
        """Test to see if value is in the range

        If the inverse flag is set (max:min range given) then this tests to
        see if the value is outside the range

        """

        log = logging.getLogger(
            f"{__name__}.{__class__.__name__}.__contains__"
        )
        log.debug(
            "low=%r high=%r value=%r inverse=%r",
            self._low_bounds,
            self._high_bounds,
            value,
            self._inverse,
        )
        if not isinstance(value, self.CONV):
            raise TypeError("Comparison to invalid type {type(value)}")

        in_it = True
        if value > self._high_bounds:
            in_it = False
        if self._low_bounds > value:
            in_it = False
        if self._inverse:
            in_it = not in_it
        return in_it

    @property
    def spec(self) -> str:
        """Return the original range spec given"""
        return self._range_spec

    @property
    def low(self) -> int:
        """Low bounds"""
        return self._low_bounds

    @property
    def high(self) -> int:
        """High bounds"""
        return self._high_bounds

    @property
    def inverse(self) -> int:
        """Get the status of the inverse flag"""
        return self._inverse


class FRange(Range):
    """Floating point version of Range"""

    RE_RANGE = re.compile(
        "^@"
        "(?P<lval>-?(?:[0-9]+[.])?[0-9]+)?"
        ":"
        "(?P<rval>-?(?:[0-9]+[.])?[0-9]+)?"
        "$"
    )
    RE_SINGLE = re.compile("^(?:[0-9]+[.])?[0-9]+$")
    CONV = float
    MIN = 0.0
    MAX = float(2**32 - 1)

    @property
    def low(self) -> float:
        """Low bounds"""
        return self._low_bounds

    @property
    def high(self) -> float:
        """High bounds"""
        return self._high_bounds


def units(value: int | float) -> str:
    """Convert int or float into a number with kB/GB/TB etc after it"""

    labels = ["kB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB"]
    label = ""
    valcopy = value
    try:
        while valcopy > 1024:
            valcopy /= 1024
            label = labels.pop(0)
    except IndexError:
        valcopy = value
        label = ""
    return f"{valcopy:0.1f} {label}".strip()


if __name__ == "__main__":
    x = Range("1")



#!/usr/bin/env python3
# Copyright 2020 Hoplite Industries, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Disk checks."""

import platform

if platform.system() != "Windows":
    import pwd

# 3rd party
import psutil

# Local imports
import libnagios

TEMPLATE = """Processes {check} {status} :: {prefix}{msg}{postfix}{range}"""


class Check(libnagios.plugin.Plugin):
    """Nagios plugin to perform process checks."""

    EPILOG = libnagios.doc.RANGE_DOC

    def init(self):
        """Initialize things that are needed in this module"""
        # pylint: disable=attribute-defined-outside-init
        self._user_set = False
        self._user = None

    def cli(self):
        """Add command line arguments specific to the plugin."""
        self.parser.add_argument(
            "-w",
            "--warn",
            dest="warn",
            metavar="<range>",
            type=libnagios.utils.Range,
            default=libnagios.utils.Range("1"),
            help=(
                "Processes outside the range spec flag warning. See range "
                "spec below for details on what this means.  [Default: 1]"
            ),
        )
        self.parser.add_argument(
            "-c",
            "--critical",
            dest="critical",
            metavar="<range>",
            type=libnagios.utils.Range,
            default=libnagios.utils.Range("1"),
            help=(
                "Processes outside the range spec flag critical. See range "
                "spec below for details on what this means.  [Default: 1]"
            ),
        )
        self.parser.add_argument(
            "-m",
            "--metric",
            choices=("PROCS", "VSZ", "RSS"),
            default="PROCS",
            dest="metric",
            help=(
                "Choose which metric to do the comparison against: "
                "[Default: %(default)s]"
            ),
        )
        self.parser.add_argument(
            "-C",
            "--command",
            dest="command",
            metavar="<command>",
            default=None,
            help="Only scan for exact matches of <command> (without path).",
        )
        self.parser.add_argument(
            "-u",
            "--user",
            dest="user",
            metavar="<user>",
            default=None,
            help="Only scan processes belonging to user",
        )
        self.parser.add_argument(
            "-r",
            "--rss",
            dest="rss",
            metavar="<rss>",
            type=int,
            default=None,
            help="Only scan for processes with RSS higher than indicated.",
        )
        self.parser.add_argument(
            "-z",
            "--vsz",
            dest="vsz",
            metavar="<vsz>",
            type=int,
            default=None,
            help="Only scan for processes with VSZ higher than indicated.",
        )

    def skip_proc(self, proc: psutil.Process) -> bool:
        """Apply base filters to processes

        Returns:
            True if the process should be skiped due to common rules being
            triggered.  False if the process should not be filtered out.

        """
        skip = False
        if not skip and self.opts.command and proc.name() != self.opts.command:
            skip = True

        try:
            if not skip and self.user and proc.username() != self.user:
                skip = True
        except psutil.AccessDenied as err:
            raise libnagios.exceptions.UnknownError(
                f"Insufficient permissions to check processes ownership: {err}"
            ) from None

        if not skip:
            meminfo = proc.memory_info()

            if (
                not skip
                and self.opts.rss is not None
                and meminfo.rss < self.opts.rss
            ):
                # Skip processes that don't meet the minimum RSS size
                skip = True

            if (
                not skip
                and self.opts.vsz is not None
                and meminfo.vms < self.opts.vsz
            ):
                # Skip processes that don't meet the minimum VSZ size
                skip = True
        return skip

    def process_procs(self, state: dict[str, str]):
        """Process PROCS metrics flavor

        Parameters:
            state: state dictionary from the execute function

        """
        # Filter processes based on criteria
        count = 0
        for proc in psutil.process_iter():
            with proc.oneshot():
                if self.skip_proc(proc):
                    continue

                # increment various counters
                count += 1
        state["msg"] = f"{count} processes"
        if count not in self.opts.warn:
            self.status = libnagios.types.Status.WARN
            word = "inside" if self.opts.warn.inverse else "outside"
            state["range"] = (
                f" - {word} range {self.opts.warn.low} "
                f"to {self.opts.warn.high}"
            )
        if count not in self.opts.critical:
            self.status = libnagios.types.Status.CRITICAL
            word = "inside" if self.opts.critical.inverse else "outside"
            state["range"] = (
                f" - {word} range {self.opts.critical.low} "
                f"to {self.opts.critical.high}"
            )

    def process_vsz_rss(self, flavor: str, state: dict[str, str]):
        """Process VSZ and RSS metrics flavor

        Parameters:
            flavor: Either string 'vsz' or 'rss'
            state: state dictionary from the execute function

        """
        # Filter processes based on criteria
        errors = []
        for proc in psutil.process_iter():
            with proc.oneshot():
                if self.skip_proc(proc):
                    continue

                # increment various counters
                meminfo = proc.memory_info()
                match flavor:
                    case "vsz":
                        value = getattr(meminfo, "vms")
                    case "rss":
                        value = getattr(meminfo, "rss")
                    case _:
                        raise ValueError(
                            f"flavor must be either vsz or rss not {flavor}"
                        )
                comps = [
                    (libnagios.types.Status.CRITICAL, self.opts.critical),
                    (libnagios.types.Status.WARN, self.opts.warn),
                ]

                for status, comp_range in comps:
                    if value not in comp_range:
                        self.status = (
                            status if self.status < status else self.status
                        )
                        word = "inside" if comp_range.inverse else "outside"
                        value_str = libnagios.utils.units(value)
                        low_str = libnagios.utils.units(comp_range.low)
                        high_str = libnagios.utils.units(comp_range.high)
                        errors.append(
                            f" {status.name} pid[{proc.pid}] {proc.name()} "
                            f"[{value_str}] {word} range {low_str} to "
                            f"{high_str}"
                        )
                        break
        state["range"] = "\n".join(errors)
        state["msg"] = f"{len(errors)} processes"

    @property
    def user(self):
        """Uset attribute for the purposes of filtering.  None is no filter"""
        # pylint: disable=attribute-defined-outside-init
        if not self._user_set:
            if self.opts.user:
                if platform.system() == "Windows":
                    # Don't do validation on windows.... use as is
                    self._user = self.opts.user
                else:
                    try:
                        # Test for user being a uid
                        uid = int(self.opts.user)
                        self._user = pwd.getpwuid(uid).pw_name
                    except ValueError:
                        # not a uid.  Must be a user name instead
                        try:
                            self._user = pwd.getpwnam(self.opts.user).pw_name
                        except KeyError:
                            raise libnagios.exceptions.UnknownError(
                                f"Invalid user [{self.opts.user}]"
                            ) from None
                    except KeyError:
                        raise libnagios.exceptions.UnknownError(
                            f"Invalid uid [{self.opts.user}]"
                        ) from None
            else:
                self._user = None
            self._user_set = True
        return self._user

    def execute(self):
        """Execute the actual working parts of the plugin."""

        # Default values for template
        state = {
            "prefix": (
                f"Command {self.opts.command} has "
                if self.opts.command
                else ""
            ),
            "postfix": f" for user {self.user}" if self.user else "",
            "range": "",
            "msg": "OK",
            "status": self.status.name,
        }

        # Start checking stuff
        match self.opts.metric:
            case "PROCS":
                self.process_procs(state)
            case "VSZ":
                self.process_vsz_rss("vsz", state)
            case "RSS":
                self.process_vsz_rss("rss", state)
            case _:
                state["msg"] = f"Unhandled metric type {self.opts.metric}"
                self.status = libnagios.types.Status.UNKNOWN

        state["status"] = self.status.name
        state["check"] = self.opts.metric
        self.message = TEMPLATE.strip().format(**state)


def run():
    """Entry point from setup.py for installation of wrapper."""
    instance = Check()
    instance.main()


if __name__ == "__main__":
    run()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Documentation types used by libnagios

The docs here are meant to be formatted with a markdown formatter.
"""

# Documentation for ranges specified on the cli
RANGE_DOC = """
# RANGES

Ranges are specified in two ways.  First is via a single value which
represents the upper bounds of a range that starts at 0.  The second
is where you can specify the upper and lower bounds of the range in
one argument.

RANGEs are prefixed with @ and specified **@min:max**, **@min:** or
**@:max**.  If specified **max:min**, then you are testing for things
outside of the specified range.

Examples:

* **@10:30**    Tests for membership in range 10 - 30 inclusive.
* **@10:**      Tests for membership in range 10 - 2^32-1 inclusive.
* **@:30**      Tests for membership in range 0 - 30 inclusive.
* **@30:10**    Tests for membership outside the range 10 - 30 (9 and
    below or 31 and above)
"""



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Exceptions for libnagios"""

import logging

# Local imports
from . import types

logging.getLogger(__name__).addHandler(logging.NullHandler())


class NagiosException(Exception):
    """Base exception for libnagios"""

    STATUS = types.Status.UNKNOWN

    def __init__(self, message: str, perfdata: None | dict[str, any] = None):
        self._message = message
        self._perfdata = perfdata
        super().__init__(message)

    @property
    def message(self) -> str:
        """Message property"""
        return self._message

    @property
    def perfdata(self) -> dict[str, any]:
        """Performance data if set"""
        return {} if self._perfdata is None else self._perfdata

    @property
    def status(self) -> types.Status:
        """Nagios Status code"""
        return self.STATUS


class OKError(NagiosException):
    """Short circuit way to generate an OK status"""

    STATUS = types.Status.OK


class WarnError(NagiosException):
    """Short circuit way to generate an WARN status"""

    STATUS = types.Status.WARN


class CriticalError(NagiosException):
    """Short circuit way to generate an CRITICAL status"""

    STATUS = types.Status.WARN


class UnknownError(NagiosException):
    """Raised when an UNKNOWN condition exists"""

    STATUS = types.Status.UNKNOWN



#!/usr/bin/env python3
# Copyright 2020 Hoplite Industries, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Disk checks."""

import dbm
import json
import os.path
import platform
import sys
import time

# 3rd party
import psutil

# Local imports
import libnagios

TEMPLATE = """CPU Usage  {swap_free:,.3f} GB ({swap_free_pct:.2%})
Swap Total: {swap_total:,.3f} GB
Swap Used: {swap_used:,.3f} GB ({swap_used_pct:.2f})
"""


class ReturnErr(Exception):
    """Exception indicating the error message to return"""

    def __init__(self, message, status):
        super().__init__(message)
        self.message = message
        self.status = status


# pylint: disable=consider-using-generator
class Check(libnagios.plugin.Plugin):
    """Nagios plugin to perform CPU checks."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.current = psutil.cpu_times()

    def cli(self):
        """Add command line arguments specific to the plugin."""
        # Default config.ini path
        winpath = os.path.join(
            os.path.dirname(os.path.realpath(sys.argv[0])), "check_cp_cpu.db"
        )
        posixpath = os.path.expanduser("~/.check_cp_cpu.db")
        cfgpath = winpath if platform.system() == "Windows" else posixpath

        self.parser.add_argument(
            "-t",
            "--time-span",
            dest="span",
            type=int,
            default=300,
            help="Time in seconds to do calculations for.  This value should "
            "be some multiple of the frequency of your checks. [Default: "
            "%(default)d]",
        )
        self.parser.add_argument(
            "-s",
            "--state-file",
            dest="state",
            default=cfgpath,
            help="Path to state database [Default: %(default)s]",
        )
        # pylint: disable=consider-using-f-string
        self.parser.add_argument(
            "-w",
            "--warn",
            dest="warn",
            nargs=2,
            action="append",
            metavar="<type> <value>",
            default=[["user", 80.0], ["system", 50.0]],
            help="CPU usage percent to warn at.  The defaults are 80 for "
            "'user' and 50 for 'system'.  Valid states are: %s"
            % ", ".join(self.current._fields),
        )
        self.parser.add_argument(
            "-c",
            "--critical",
            dest="critical",
            nargs=2,
            action="append",
            metavar="<type> <value>",
            default=[["user", 80.0], ["system", 50.0]],
            help="CPU usage percent to go critical at.  The defaults are 90 "
            "for 'user' and 75 for 'system'.  Valid states are: %s"
            % ", ".join(self.current._fields),
        )

    def get_history(self):
        """Get the history and state from the db cache"""
        try:
            statedb = dbm.open(self.opts.state, "c")
        except dbm.error as err:
            raise ReturnErr(
                f"Failed to open state db: {err}",
                libnagios.types.Status.UNKNOWN,
            ) from None

        now = int(time.time())
        old = now - self.opts.span * 3

        statedb[str(now)] = json.dumps(self.current._asdict())

        closest = None
        for key in list(statedb.keys()):
            timestamp = int(key.decode("utf-8"))

            # Cleanup of old stuff
            if timestamp < old:
                del statedb[key]
                continue

            # Ignore current timestamp
            if timestamp == now:
                continue

            if closest is None:
                closest = timestamp
                continue

            cl_distance = abs(self.opts.span - (now - closest))

            if abs(self.opts.span - (now - timestamp)) < cl_distance:
                closest = timestamp

        if closest is None:
            raise ReturnErr(
                "Not enough data points yet..", libnagios.types.Status.UNKNOWN
            ) from None

        history = json.loads(statedb[str(closest)])
        statedb.close()
        return closest, history

    # Yes we know this is a big function.
    # pylint: disable=too-many-locals,too-many-branches
    def execute(self):
        """Execute the actual working parts of the plugin."""
        # validate types
        warn = {}
        critical = {}
        current = self.current._asdict()
        for key, value in self.opts.warn:
            if key not in current:
                # pylint: disable=consider-using-f-string
                self.message = (
                    "Invalid CPU state: [%s]. Valid values are [%s]"
                    % (
                        key,
                        ", ".join(current.keys()),
                    )
                )
                self.status = libnagios.types.Status.UNKNOWN
                return
            try:
                warn[key] = float(value)
            except ValueError:
                self.message = f"Warn value for {key} must be a float"
                self.status = libnagios.types.Status.UNKNOWN
                return

        for key, value in self.opts.critical:
            if key not in current:
                self.message = (
                    # pylint: disable=consider-using-f-string
                    "Invalid CPU state: [%s]. Valid values are [%s]"
                    % (
                        key,
                        ", ".join(current.keys()),
                    )
                )
                self.status = libnagios.types.Status.UNKNOWN
                return
            try:
                critical[key] = float(value)
            except ValueError:
                self.message = f"Warn value for {key} must be a float"
                self.status = libnagios.types.Status.UNKNOWN
                return

        now = time.time()
        try:
            closest, history = self.get_history()
        except ReturnErr as err:
            self.message = err.message
            self.status = err.status
            return

        start_ticks = sum([history[x] for x in history])
        end_ticks = sum([current[x] for x in current])
        total = end_ticks - start_ticks

        used = (
            sum(
                [
                    (current[x] - history[x])
                    for x in current
                    if x not in ("idle",)
                ]
            )
            / total
            * 100
        )

        self.message = f"ticks: {total}"

        stats = {x: ((current[x] - history[x]) / total) * 100 for x in current}
        output = []

        for key, value in critical.items():
            if stats[key] > value:
                seconds = abs(closest - now)
                output.append(
                    f"CRITICAL: {key} CPU is {stats[key]}% for the last "
                    f"{seconds} seconds"
                )
                self.status = libnagios.types.Status.CRITICAL

        if not output:
            for key, value in warn.items():
                if stats[key] > value:
                    output.append(
                        f"WARNING: {key} CPU is {stats[key]}% for the last "
                        f"{seconds} seconds"
                    )
                    self.status = libnagios.types.Status.WARN

        if not output:
            output = [f"CPU Usage OK at {used:.2f}%"]

        for key in sorted(stats.keys()):
            output.append(f"{key} CPU usage: {stats[key]:.2f}%")

        self.message = "\n".join(output)
        self.add_perf_multi({f"cpu_{x}": stats[x] for x in stats})


def run():
    """Entry point from setup.py for installation of wrapper."""
    instance = Check()
    instance.main()


if __name__ == "__main__":
    run()



#!/usr/bin/env python3
# Copyright 2022 Hoplite Industries, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""SSH network check."""

import socket
import time

# 3rd party imports
import paramiko

# local imports
import libnagios

TEMPLATE = """{bit} bit {type} ssh2 connection established
Fingerprint: {fingerprint}
Public Key: {pubkey}
In {elapsed} seconds.
"""


class Check(libnagios.plugin.Plugin):
    """Nagios plugin to perform SSH connectivity check."""

    def cli(self):
        """Add command line arguments specific to the plugin."""
        self.parser.set_defaults(afinet=0)
        group = self.parser.add_mutually_exclusive_group()
        group.add_argument(
            "-4",
            "--ipv4",
            dest="afinet",
            action="store_const",
            const=socket.AF_INET,
            help="Only do IPv4",
        )
        group.add_argument(
            "-6",
            "--ipv6",
            dest="afinet",
            action="store_const",
            const=socket.AF_INET6,
            help="Only do IPv6",
        )
        self.parser.add_argument(
            "-p",
            "--port",
            dest="port",
            default=22,
            type=int,
            help="SSH port [Default: %(default)d]",
        )
        self.parser.add_argument(
            "-H",
            "--host",
            dest="host",
            default="localhost",
            help="SSH Host [Default: %(default)s]",
        )
        self.parser.add_argument(
            "-w",
            "--warn",
            dest="warn",
            type=float,
            default=5.0,
            help="Go warning after X seconds [Default: %(default)0.2f]",
        )
        self.parser.add_argument(
            "-c",
            "--critical",
            dest="critical",
            type=float,
            default=10.0,
            help="Go critical after X seconds [Default: %(default)0.2f]",
        )

    def execute(self):
        """Execute the actual working parts of the plugin."""

        try:
            possible = socket.getaddrinfo(
                self.opts.host,
                self.opts.port,
                family=self.opts.afinet,
                type=socket.SOCK_STREAM,
            )
        except socket.gaierror as err:
            self.message = f"{self.opts.host}:{self.opts.port} {str(err)}"
            self.status = libnagios.types.Status.CRITICAL
            return

        for family, sock_type, proto, _, sockaddr in possible:
            start = time.time()
            sock = socket.socket(family=family, type=sock_type, proto=proto)
            sock.settimeout(self.opts.critical)
            try:
                self.status = libnagios.types.Status.OK
                sock.connect(sockaddr)
                conn = paramiko.transport.Transport(sock)
                conn.start_client(timeout=self.opts.critical)
                if not conn.is_active():
                    self.message = "SSH session inactive error"
                    self.status = libnagios.types.Status.CRITICAL

                key = conn.get_remote_server_key()
                conn.close()

                elapsed = round(time.time() - start, 3)

                self.message = TEMPLATE.format(
                    bit=key.get_bits(),
                    type=key.get_name(),
                    fingerprint=key.get_fingerprint().hex(),
                    elapsed=elapsed,
                    pubkey=key.get_base64(),
                )

                # Check thresholds for timeouts
                if elapsed > self.opts.warn:
                    self.message = f"Timeout after {elapsed:.2f} seconds"
                    self.status = libnagios.types.Status.WARN
                if elapsed > self.opts.critical:
                    self.message = f"Timeout after {elapsed:.2f} seconds"
                    self.status = libnagios.types.Status.CRITICAL
            except paramiko.ssh_exception.SSHException as err:
                self.message = f"Error: {str(err)}"
                self.status = libnagios.types.Status.CRITICAL
            except socket.timeout:
                self.message = f"Timeout after {elapsed:.2f} seconds"
                self.status = libnagios.types.Status.CRITICAL
            except socket.gaierror as err:
                self.message = f"Socket error: {err}"
                self.status = libnagios.types.Status.CRITICAL
            except OSError as err:
                self.message = f"Generic Socket error: {err}"
                self.status = libnagios.types.Status.CRITICAL
            finally:
                elapsed = time.time() - start
                self.add_perf_multi({"elapsed": round(time.time() - start, 3)})
            return


def run():
    """Entry point from setup.py for installation of wrapper."""
    instance = Check()
    instance.main()


if __name__ == "__main__":
    run()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Plugin template."""

import argparse
import base64
import io
import json
import logging
import os
import sys
import time
import typing

# 3rd party imports
import rich.markdown
import rich_argparse

# Local imports
from . import exceptions
from . import types

logging.getLogger(__name__).addHandler(logging.NullHandler())


class Plugin:
    """This class is a template for a nagios plugin.

    Attributes:
        parser (argparse.ArgumentParser): To be used in your subclasses as
            to add command line arugments within your :ref:`Plugin.cli` method.
        opts (argparse.Namespace): The options returned from parsing the
            command line arguments.  To be used in your :ref:`Plugin.execute`
            method.

    """

    DESCRIPTION = None
    EPILOG = None

    def __init__(self):
        log = logging.getLogger(f"{__name__}.Plugin")
        log.debug("Initilization")

        # Public variables
        self.parser = None
        self.opts = None

        self._message = None
        self._status = types.Status.OK
        self._perfdata = {}
        self.init()

    def init(self):
        """Meant to be overridden if your module needs additinal setup"""

    @property
    def message(self):
        """(str) Status message to give to Nagios.

        This can be a multi-line output string.  The first line is what
        Nagios will display when looking at the summary.  The details page will
        show the full output.

        .. note::

            This value may not have a pipe (|) character in it.

        """
        return self._message

    @message.setter
    def message(self, value):
        if not isinstance(value, str):
            raise ValueError("'message' must be a 'str'")
        if "|" in value:
            raise ValueError(
                "'message' must not contain the pipe '|' character"
            )
        self._message = value.strip()

    @property
    def status(self):
        """(Status) Status of the plugin run.

        Defaults to :ref:`OK <Status>`
        """
        return self._status

    @status.setter
    def status(self, value):
        if not isinstance(value, types.Status):
            raise ValueError(
                "'status' must be an instance of 'libnagios.types.Status'"
            )
        self._status = value

    def add_perf(self, key: str, value: typing.Union[str, float, int]):
        """Add performance data to the Nagios output.

        Only str, float or int values may be added.

        Parameters:
            key: Key name for the performance data.  There are three keys
                that are reserved and will be overridden by the
                :ref:`Plugin.main` method.

                * ``epoch``: Time in seconds since 00:00:00 Jan 1, 1970 GMT.
                * ``runtime``: Time it took to run the :meth:`Plugin.execute`
                    method from your subclass.
                * ``state``: String value for the state of the plugin run.

            value (str, float, or int): Performance metric value to record.

        """
        log = logging.getLogger(f"{__name__}.Plugin.add_perf")
        log.debug("%s=%s", repr(key), repr(value))
        if not isinstance(key, str):
            raise ValueError("When adding perf data 'key' must be a 'str'")

        if not isinstance(key, (str, float, int)):
            raise ValueError(
                f"For key={repr(key)} 'value' must in ('str', 'float', 'int')"
            )

        self._perfdata[key] = value

    def add_perf_multi(self, data: dict):
        """Add performance data to the Nagios output.

        Add the contents of the dictionary to the perfdata output of your
        plugin.

        Parameters:
            data: Key value pairs added to the perfdata output of the plugin.
                each key/value pair is sent through
                :meth:`libnagios.plugin.Plugin.add_perf` to add the data.

        Raises:
            ValueError: Raised on multiple conditions:

                * Raised when ``data`` is not a ``dict`` instance.
                * Raised when a key in the dictionary is not a ``str``
                * Raised when a value in the dictionary is not one of ``str``,
                  ``float``, ``int``.

        .. note::

            See :meth:`libnagios.plugin.Plugin.add_perf` for details on
            reserved keys.
        """

        log = logging.getLogger(f"{__name__}.Plugin.add_perf_multi")
        log.debug("data is {type(data)}")

        if not isinstance(data, dict):
            raise ValueError(
                "When adding multiple values 'data' must be a 'dict'"
            )

        for key, value in data.items():
            self.add_perf(key, value)

    def finish(self, as_json: bool = False, limit: int = 4096):
        """Print plugin output for Nagios and exit.

        Last method called by :meth:`libnagios.plugin.Plugin.main` to print
        the nagios output (message and perfdata).  Then it exits with the
        appropriate exit status from :meth:`libnagios.plugin.Plugin.status`

        Parameters:
            as_json: If ``True`` print the perfdata as JSON.  If ``False``
                (default) print the perf data as ``key=value`` one line per
                key.
            limit: The amount of output to print.  By defult Nagios will only
                take the first 4k of output, anything after that will be
                discarded.  If you compiled your Nagios to handle more
                than 4k you may adjust ``limit`` to accommodate that.

        """
        log = logging.getLogger(f"{__name__}.Plugin.finish")
        log.debug("(as_json=%s, limit=%s)", repr(as_json), repr(limit))
        output = io.StringIO()

        if self._message:
            output.write(self._message)
        else:
            output.write(f"CRITICAL {__name__}.Plugin.message udefined....")
            self._status = types.Status.CRITICAL
        if self._perfdata:
            output.write("|")

            if as_json:
                output.write(json.dumps(self._perfdata))
            else:
                perfdata = []
                for key, value in self._perfdata.items():
                    perfdata.append(f"{key}={str(value)}")
                output.write("\n".join(perfdata))
        output.seek(0)
        sys.stdout.write(output.read(limit))
        sys.exit(self._status.value)

    def cli(self):
        """Override me.

        Class to be overridden in a subclass to handle command line arguments
        for your plugin.  You may use the
        :attr:`libnagios.plugin.Plugin.parser` variable to add command line
        arguments.

        """
        raise RuntimeError("You must implement Plugin.cli in subclass")

    def execute(self):
        """Override me.

        Class to be overridden in a subclass to do the actual work for your
        plugin.  You may access the command line options via the
        :attr:`libnagios.plugin.Plugin.opts` variable.

        """
        raise RuntimeError("You must implement Plugin.execute in subclass")

    def _parse_args(self):
        random_session = base64.urlsafe_b64encode(os.urandom(9)).decode(
            "utf-8"
        )
        session_id = os.getenv("SESSION_ID", random_session)

        epilog = (
            rich.markdown.Markdown(self.EPILOG, style="argparse.text")
            if self.EPILOG
            else None
        )
        description = (
            rich.markdown.Markdown(self.DESCRIPTION, style="argparse.text")
            if self.DESCRIPTION
            else None
        )
        self.parser = argparse.ArgumentParser(
            formatter_class=rich_argparse.RichHelpFormatter,
            description=description,
            epilog=epilog,
        )

        self.parser.add_argument(
            "--debug",
            dest="debug",
            default=False,
            action="store_true",
            help="Turn on debug output",
        )
        self.parser.add_argument(
            "--log",
            dest="debug_log",
            default=None,
            help="Specify a file for debug output. Implies --debug",
        )
        self.parser.add_argument(
            "--verbose",
            dest="verbose",
            default=False,
            action="store_true",
            help="Turn on verbose output",
        )
        self.parser.add_argument(
            "--session-id",
            dest="session_id",
            default=session_id,
            help="Specify a session id for logging",
        )
        self.parser.add_argument(
            "--json",
            dest="as_json",
            default=False,
            action="store_true",
            help="Output perfdata as json",
        )

        self.cli()
        self.opts = self.parser.parse_args()
        if self.opts.debug_log:
            self.opts.debug = True

    def main(self):
        """Entry point into the class and it's subclasses.

        When using the ``Plugin`` class to create a nagios plugin you will
        need to use this method as the primary entry point.  This class will
        perform the checks and report back the proper status to the calling
        Nagios entity.

        .. note::

            This function exits the program, so make sure no vital code is
            executed after the call to ``main()``.

        """
        self._parse_args()
        handlers = []
        # Set up stderr logging
        stderr = logging.StreamHandler(stream=sys.stderr)
        handlers.append(stderr)

        # Set the defaults
        fmt = "%(asctime)s %(levelname)s %(name)s: %(message)s"
        level = logging.CRITICAL
        args = {}

        if self.opts.debug:
            level = logging.DEBUG
            if self.opts.debug_log:
                args = {
                    "filename": self.opts.debug_log,
                    "filemode": "a",
                }
        elif self.opts.verbose:
            level = logging.INFO
        else:
            fmt = "%(name)s: %(message)s"
        logging.basicConfig(level=level, format=fmt, handlers=handlers, **args)
        log = logging.getLogger()
        log.setLevel(level)

        start = time.time()
        try:
            self.execute()
        except exceptions.NagiosException as err:
            self.message = err.message
            self.add_perf_multi(err.perfdata)
            self.status = err.status
        runtime = time.time() - start
        self.add_perf("epoch", start)
        self.add_perf("runtime", f"{runtime:.2f}")
        self.add_perf("state", self.status.name)
        self.finish(as_json=self.opts.as_json)



#!/usr/bin/env python3
# Copyright 2020 Hoplite Industries, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Disk checks."""

import shutil

# Local imports
import libnagios

TEMPLATE = """Disk {disk} :: Free: {disk_free:,.3f} GB ({disk_free_pct:.2f})
Disk Total: {disk_total:,.3f} GB
Disk Used: {disk_used:,.3f} GB ({disk_used_pct:.2f})
"""


class Check(libnagios.plugin.Plugin):
    """Nagios plugin to perform Disk checks."""

    def cli(self):
        """Add command line arguments specific to the plugin."""
        group = self.parser.add_mutually_exclusive_group()
        group.add_argument(
            "-p",
            "--percent",
            dest="percent",
            action="store_true",
            default=False,
            help="Warning/Critical values are a percentage (default)",
        )
        group.add_argument(
            "-m",
            "--mega-bytes",
            dest="mb",
            action="store_true",
            default=False,
            help="Warning/Critical values are in Mega-Bytes",
        )
        group.add_argument(
            "-g",
            "--giga-bytes",
            dest="gb",
            action="store_true",
            default=False,
            help="Warning/Critical values are in Giga-Bytes",
        )
        self.parser.add_argument(
            "-w",
            "--warn",
            dest="warn",
            type=float,
            default=20.0,
            help="Amount of disk free to warn at",
        )
        self.parser.add_argument(
            "-c",
            "--critical",
            dest="critical",
            type=float,
            default=10.0,
            help="Amount of disk free to mark critical "
            "[Default: %0.2(default)f]",
        )
        self.parser.add_argument(
            "disk",
            help="Directory path for disk to check",
        )

    def execute(self):
        """Execute the actual working parts of the plugin."""
        try:
            result = shutil.disk_usage(self.opts.disk)
        except OSError as err:
            self.message = f"Error gathering disk usage: {err}"
            self.status = libnagios.types.Status.UNKNOWN
            return

        # Stats and stuff
        stats = {
            "disk": self.opts.disk,
            "disk_total": result.total / (1024 * 1024 * 1024),
            "disk_used": result.used / (1024 * 1024 * 1024),
            "disk_free": result.free / (1024 * 1024 * 1024),
            "disk_free_pct": (result.free / result.total) * 100.0,
            "disk_used_pct": (result.used / result.total) * 100.0,
        }

        if self.opts.mb or self.opts.gb:
            divisor = 1024 * 1024 if self.opts.mb else 1024 * 1024 * 1024
            free = result.free / divisor
        else:
            # Fallback to percentage
            free = (result.free / result.total) * 100.0

        if free < self.opts.critical:
            self.status = libnagios.types.Status.CRITICAL
        elif free < self.opts.warn:
            self.status = libnagios.types.Status.WARN
        else:
            self.status = libnagios.types.Status.OK

        self.message = TEMPLATE.strip().format(**stats)
        self.add_perf_multi(stats)


def run():
    """Entry point from setup.py for installation of wrapper."""
    instance = Check()
    instance.main()


if __name__ == "__main__":
    run()



#!/usr/bin/env python3
# Copyright 2020 Hoplite Industries, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Bitlocker checks."""

import os.path
import subprocess

import libnagios

TEMPLATE = """Disk {disk} :: Bitlocker State: {state}
{stdout}
"""


class Check(libnagios.plugin.Plugin):
    """Nagios plugin to perform Disk checks."""

    def cli(self):
        """Add command line arguments specific to the plugin."""
        self.parser.add_argument(
            "disk",
            help="Directory path for disk to check",
        )

    def execute(self):
        """Execute the actual working parts of the plugin."""
        try:
            cmd = [
                os.path.join(
                    "C:", os.sep, "Windows", "system32", "manage-bde.exe"
                ),
                "-status",
                self.opts.disk,
                "-ProtectionAsErrorLevel",
            ]
            # pylint: disable=subprocess-run-check
            exe = subprocess.run(cmd, capture_output=True)
        except OSError as err:
            self.message = f"Error gathering disk usage: {err}"
            self.status = libnagios.types.Status.UNKNOWN
            return

        # Stats and stuff
        stats = {
            "disk": self.opts.disk,
            "state": "Active" if exe.returncode == 0 else "Inactive",
            "stdout": exe.stdout.decode("utf-8"),
        }

        self.status = libnagios.types.Status.CRITICAL
        if exe.returncode == 0:
            self.status = libnagios.types.Status.OK

        self.message = TEMPLATE.strip().format(**stats)


def run():
    """Entry point from setup.py for installation of wrapper."""
    instance = Check()
    instance.main()


if __name__ == "__main__":
    run()



#!/usr/bin/env python3
# Copyright 2020 Hoplite Industries, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Disk checks."""

import platform
import time

# 3rd party
import psutil

# Local imports
import libnagios

TEMPLATE = """Load {status} :: {msg}
One minute:      {one:.1f}
Five minute:     {five:.1f}
Fifteen minute:  {fifteen:.1f}
"""


class Check(libnagios.plugin.Plugin):
    """Nagios plugin to perform Load checks."""

    def cli(self):
        """Add command line arguments specific to the plugin."""
        warn = [12, 10, 8]
        crit = [15, 12, 10]
        self.parser.add_argument(
            "-w",
            "--warn",
            dest="warn",
            type=float,
            nargs=3,
            default=warn,
            help="Load average to warn at. [Defaults: {:.1f}, "
            "{:.1f}, {:.1f} for 1, 5, 15 minute average.]".format(*warn),
        )
        self.parser.add_argument(
            "-c",
            "--critical",
            dest="critical",
            type=float,
            nargs=3,
            default=crit,
            help="Load average to go critical at. [Defaults: {:.1f}, "
            "{:.1f}, {:.1f} for 1, 5, 15 minute average.]".format(*crit),
        )

    def execute(self):
        """Execute the actual working parts of the plugin."""
        try:
            stats = dict(zip(["one", "five", "fifteen"], psutil.getloadavg()))
            if platform.system() == "Windows":
                time.sleep(5.5)
                stats = dict(
                    zip(["one", "five", "fifteen"], psutil.getloadavg())
                )
        except OSError as err:
            self.message = f"Error gathering load average: {err}"
            self.status = libnagios.types.Status.UNKNOWN
            return

        self.add_perf_multi({f"loadavg_{x}": stats[x] for x in stats})

        output = {}
        for status, values in (
            (libnagios.types.Status.WARN, self.opts.warn),
            (libnagios.types.Status.CRITICAL, self.opts.critical),
        ):
            output[status] = []
            one, five, fifteen = values
            if stats["one"] > one:
                self.status = status
                output[status].append("1 min: {stats['one']:.1f} > {one:.1f}")
            if stats["five"] > five:
                self.status = status
                output[status].append(
                    "5 min: {stats['five']:.1f} > {five:.1f}"
                )
            if stats["fifteen"] > fifteen:
                self.status = status
                output[status].append(
                    "15 min: {stats['fifteen']:.1f} > {fifteen:.1f}"
                )

        # pylint: disable=consider-using-f-string
        stats["msg"] = "{one:.1f}, {five:.1f}, {fifteen:.1f}".format(**stats)

        # Order matters.  Highest criticality must be last
        for status in (
            libnagios.types.Status.WARN,
            libnagios.types.Status.CRITICAL,
        ):
            if output[status]:
                self.status = status
                stats["msg"] = " :: ".join(output[status])

        stats["status"] = self.status.name
        self.message = TEMPLATE.strip().format(**stats)


def run():
    """Entry point from setup.py for installation of wrapper."""
    instance = Check()
    instance.main()


if __name__ == "__main__":
    run()
